@@ -3,17 +3,17 @@ doctr
 
 A tool to automatically deploy docs to GitHub pages from Travis CI.
 
-The doctr command is two commands in one. To use, first run
+The doctr command is two commands in one. To use, first run::
 
-doctr configure
+    doctr configure
 
 on your local machine. This will prompt for your GitHub credentials and the
 name of the repo you want to deploy docs for. This will generate a secure key,
 which you should insert into your .travis.yml.
 
-Then, on Travis, for the build where you build your docs, add
+Then, on Travis, for the build where you build your docs, add::
 
-    - doctr deploy
+    - doctr deploy . --built-docs path/to/built/html/
 
 to the end of the build to deploy the docs to GitHub pages.  This will only
 run on the master branch, and won't run on pull requests.
@@ -25,19 +25,21 @@ import sys
 import os
 import os.path
 import argparse
-import shlex
 import subprocess
 import yaml
 import json
+import shlex
 
 from pathlib import Path
 
 from textwrap import dedent
 
 from .local import (generate_GitHub_token, encrypt_variable, encrypt_file,
-    upload_GitHub_deploy_key, generate_ssh_key, check_repo_exists, GitHub_login)
+    upload_GitHub_deploy_key, generate_ssh_key, check_repo_exists,
+    GitHub_login, guess_github_repo)
 from .travis import (setup_GitHub_push, commit_docs, push_docs,
-    get_current_repo, sync_from_log, find_sphinx_build_dir, run, get_travis_branch)
+    get_current_repo, sync_from_log, find_sphinx_build_dir, run,
+    get_travis_branch, copy_to_tmp, checkout_deploy_branch, red)
 from . import __version__
 
 def make_parser_with_config_adder(parser, config):
@@ -132,8 +134,9 @@ options available.
     deploy_parser_add_argument('--key-path', default='github_deploy_key.enc',
         help="""Path of the encrypted GitHub deploy key. The default is %(default)r.""")
     deploy_parser_add_argument('--built-docs', default=None,
-        help="""Location of the built html documentation to be deployed to
-        gh-pages. If not specified, Doctr will try to automatically detect build location""")
+        help="""Location of the built html documentation to be deployed to gh-pages. If not
+        specified, Doctr will try to automatically detect build location
+        (right now only works for Sphinx docs).""")
     deploy_parser.add_argument('--deploy-branch-name', default=None,
                                help="""Name of the branch to deploy to (default: 'master' for ``*.github.io``
                                repos, 'gh-pages' otherwise)""")
@@ -143,8 +146,9 @@ options available.
         deploy the docs to. By default, it deploys to the repo Doctr is run from.""")
     deploy_parser_add_argument('--no-require-master', dest='require_master', action='store_false',
         default=True, help="""Allow docs to be pushed from a branch other than master""")
-    deploy_parser_add_argument('--command', default=None, help="""Command to
-        be run before committing and pushing. If the command creates
+    deploy_parser_add_argument('--command', default=None,
+        help="""Command to be run before committing and pushing. This command
+        will be run from the deploy repository/branch. If the command creates
         additional files that should be deployed, they should be added to the
         index.""")
     deploy_parser_add_argument('--no-sync', dest='sync', action='store_false',
@@ -152,6 +156,9 @@ options available.
         conjunction with the --command flag, for instance, if the command syncs
         the files for you. Any files you wish to commit should be added to the
         index.""")
+    deploy_parser.add_argument('--no-temp-dir', dest='temp_dir',
+        action='store_false', default=True, help="""Don't copy the
+        --built-docs directory to a temporary directory.""")
     deploy_parser_add_argument('--no-push', dest='push', action='store_false',
         default=True, help="Run all the steps except the last push step. "
         "Useful for debugging")
@@ -161,6 +168,7 @@ options available.
         the first argument to 'doctr deploy'. This flag is kept for backwards
         compatibility.""")
 
+
     if config:
         print('Warning, The following options in `.travis.yml` were not recognized:\n%s' % json.dumps(config, indent=2))
 
@@ -218,7 +226,7 @@ def on_travis():
 def deploy(args, parser):
     if not args.force and not on_travis():
         parser.error("doctr does not appear to be running on Travis. Use "
-            "doctr deploy --force to run anyway.")
+                     "doctr deploy <target-dir> --force to run anyway.")
 
     config = get_config()
 
@@ -242,21 +250,29 @@ def deploy(args, parser):
     if args.deploy_branch_name:
         deploy_branch = args.deploy_branch_name
     else:
-        deploy_branch = 'master' if deploy_dir.endswith(('.github.io', '.github.com')) else 'gh-pages'
+        deploy_branch = 'master' if deploy_repo.endswith(('.github.io', '.github.com')) else 'gh-pages'
 
     current_commit = subprocess.check_output(['git', 'rev-parse', 'HEAD']).decode('utf-8').strip()
     try:
         branch_whitelist = {'master'} if args.require_master else set(get_travis_branch())
         branch_whitelist.update(set(config.get('branches',set({}))))
 
-        can_push = setup_GitHub_push(deploy_repo, deploy_branch=deploy_branch,
+        canpush = setup_GitHub_push(deploy_repo, deploy_branch=deploy_branch,
                                      auth_type='token' if args.token else 'deploy_key',
                                      full_key_path=args.key_path,
                                      branch_whitelist=branch_whitelist)
 
         if args.sync:
             built_docs = args.built_docs or find_sphinx_build_dir()
+            if args.temp_dir:
+                built_docs = copy_to_tmp(built_docs)
 
+        # Reset in case there are modified files that are tracked in the
+        # dpeloy branch.
+        run(['git', 'stash', '--all'])
+        checkout_deploy_branch(deploy_branch, canpush=canpush)
+
+        if args.sync:
             log_file = os.path.join(deploy_dir, '.doctr-files')
 
             print("Moving built docs into place")
@@ -267,18 +283,25 @@ def deploy(args, parser):
             added, removed = [], []
 
         if args.command:
-            run(shlex.split(args.command))
+            run(args.command, shell=True)
 
         changes = commit_docs(added=added, removed=removed)
         if changes:
-            if can_push and args.push:
+            if canpush and args.push:
                 push_docs(deploy_branch)
             else:
                 print("Don't have permission to push. Not trying.")
         else:
             print("The docs have not changed. Not updating")
+    except BaseException as e:
+        DOCTR_COMMAND = ' '.join(map(shlex.quote, sys.argv))
+        print(red("ERROR: The doctr command %r failed: %s" % (DOCTR_COMMAND, e)),
+            file=sys.stderr)
+        raise
     finally:
-        subprocess.run(['git', 'checkout', current_commit])
+        run(['git', 'checkout', current_commit])
+        # Ignore error, won't do anything if there was nothing to stash
+        run(['git', 'stash', 'pop'], exit=False)
 
 class IncrementingInt:
     def __init__(self, i=0):
@@ -301,16 +324,35 @@ def configure(args, parser):
     else:
         login_kwargs = {'auth': None, 'headers': None}
 
-    build_repo = input("What repo do you want to build the docs for (org/reponame, like 'drdoctr/doctr')? ")
-    is_private = check_repo_exists(build_repo, service='github', **login_kwargs)
-    check_repo_exists(build_repo, service='travis')
+    get_build_repo = False
+    default_repo = guess_github_repo()
+    while not get_build_repo:
+        try:
+            if default_repo:
+                build_repo = input("What repo do you want to build the docs for [{default_repo}]? ".format(default_repo=default_repo))
+                if not build_repo:
+                    build_repo = default_repo
+            else:
+                build_repo = input("What repo do you want to build the docs for (org/reponame, like 'drdoctr/doctr')? ")
+            is_private = check_repo_exists(build_repo, service='github', **login_kwargs)
+            check_repo_exists(build_repo, service='travis')
+            get_build_repo = True
+        except RuntimeError:
+            print('\n{:-^{}}\n'.format('Invalid repo or user. Please try again.', 70))
 
-    deploy_repo = input("What repo do you want to deploy the docs to? [{build_repo}] ".format(build_repo=build_repo))
-    if not deploy_repo:
-        deploy_repo = build_repo
+    get_deploy_repo = False
+    while not get_deploy_repo:
+        try:
+            deploy_repo = input("What repo do you want to deploy the docs to? [{build_repo}] ".format(build_repo=build_repo))
+            if not deploy_repo:
+                deploy_repo = build_repo
 
-    if deploy_repo != build_repo:
-        check_repo_exists(deploy_repo, service='github', **login_kwargs)
+            if deploy_repo != build_repo:
+                check_repo_exists(deploy_repo, service='github', **login_kwargs)
+
+            get_deploy_repo = True
+        except RuntimeError:
+            print('\n{:-^{}}\n'.format('Invalid repo or user. Please try again.', 70))
 
     N = IncrementingInt(1)
 
@@ -355,38 +397,44 @@ def configure(args, parser):
 
 
         print(dedent("""\
-        {N}. Commit the file {keypath}.enc.
+        {N}. Add the file {keypath}.enc to be staged for commit:
+
+            git add {keypath}.enc
+
         """.format(keypath=args.key_path, N=N)))
 
-    options = ''
+    options = '--built-docs path/to/built/html/'
     if args.key_path != 'github_deploy_key':
         options += ' --key-path {keypath}.enc'.format(keypath=args.key_path)
     if deploy_repo != build_repo:
         options += ' --deploy-repo {deploy_repo}'.format(deploy_repo=deploy_repo)
 
     print(dedent("""\
-    {N}. Add
+    {N}. Add these lines to your `.travis.yml` file:
 
         script:
           - set -e
           - # Command to build your docs
           - pip install doctr
-          - doctr deploy{options} <deploy_directory>
-
-    to the docs build of your .travis.yml.  The 'set -e' prevents doctr from
-    running when the docs build fails. Use the 'script' section so that if
-    doctr fails it causes the build to fail.
-    """.format(options=options, N=N)))
-
-    print(dedent("""\
-    {N}. Put
+          - doctr deploy{options} <target-directory>
 
         env:
-          global:
+          global:`
             - secure: "{encrypted_variable}"
 
-    in your .travis.yml.
-    """.format(encrypted_variable=encrypted_variable.decode('utf-8'), N=N)))
+    """.format(options=options, N=N, encrypted_variable=encrypted_variable.decode('utf-8'))))
+
+    print(dedent("""\
+    {N}. Commit and push these changes to your github repository.
+        The docs should now build automatically on travis.
+
+    """.format(N=N)))
+
+    print(dedent("""\
+    Note: the `set -e` prevents doctr from running when the docs build
+      fails. We put this code under `script:` so that if doctr fails it causes
+      the build to fail.
+    """))
 
 def main():
     config = get_config()
