@@ -7,6 +7,7 @@ import json
 import uuid
 import base64
 import subprocess
+import re
 from getpass import getpass
 
 import requests
@@ -128,6 +129,12 @@ def GitHub_login(*, username=None, password=None, OTP=None, headers=None):
     if r.status_code == 401:
         two_factor = r.headers.get('X-GitHub-OTP')
         if two_factor:
+            auth_header = base64.urlsafe_b64encode(bytes(username + ':' + password, 'utf8')).decode()
+            login_kwargs = {'auth': None, 'headers': {'Authorization': 'Basic {}'.format(auth_header)}}
+            try:
+                generate_GitHub_token(**login_kwargs)
+            except requests.exceptions.HTTPError:
+                pass
             print("A two-factor authentication code is required:", two_factor.split(';')[1].strip())
             OTP = input("Authentication code: ")
             return GitHub_login(username=username, password=password, OTP=OTP, headers=headers)
@@ -248,3 +255,22 @@ def check_repo_exists(deploy_repo, service='github', *, auth=None, headers=None)
     r.raise_for_status()
 
     return r.json().get('private', False)
+
+GIT_URL = re.compile(r'(?:git@|https://|git://)github\.com[:/](.*?)(?:\.git)?')
+
+def guess_github_repo():
+    """
+    Guesses the github repo for the current directory
+
+    Returns False if no guess can be made.
+    """
+    p = subprocess.run(['git', 'ls-remote', '--get-url', 'origin'],
+        stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
+    if p.stderr or p.returncode:
+        return False
+
+    url = p.stdout.decode('utf-8').strip()
+    m = GIT_URL.fullmatch(url)
+    if not m:
+        return False
+    return m.group(1)
