@@ -9,9 +9,20 @@ import subprocess
 import sys
 import glob
 import re
+import pathlib
+import tempfile
+import time
 
 from cryptography.fernet import Fernet
 
+DOCTR_WORKING_BRANCH = '__doctr_working_branch'
+
+def red(text):
+    return "\033[31m%s\033[0m" % text
+
+def blue(text):
+    return "\033[34m%s\033[0m" % text
+
 def decrypt_file(file, key):
     """
     Decrypts the file ``file``.
@@ -78,16 +89,30 @@ def setup_deploy_key(keypath='github_deploy_key', key_ext='.enc'):
 
     run(['ssh-add', os.path.expanduser('~/.ssh/' + key_filename)])
 
-# XXX: Do this in a way that is streaming
-def run_command_hiding_token(args, token):
-    command = ' '.join(map(shlex.quote, args))
+def run_command_hiding_token(args, token, shell=False):
+    if not shell:
+        command = ' '.join(map(shlex.quote, args))
+    else:
+        command = args
     command = command.replace(token.decode('utf-8'), '~'*len(token))
-    print(command)
-    p = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    out, err = p.stdout, p.stderr
-    out = out.replace(token, b"~"*len(token))
-    err = err.replace(token, b"~"*len(token))
-    return (out, err, p.returncode)
+    print(blue(command))
+    sys.stdout.flush()
+
+    if token:
+        stdout = stderr = subprocess.PIPE
+    else:
+        stdout = stderr = None
+    p = subprocess.run(args, stdout=stdout, stderr=stderr, shell=shell)
+    if token:
+        # XXX: Do this in a way that is streaming
+        out, err = p.stdout, p.stderr
+        out = out.replace(token, b"~"*len(token))
+        err = err.replace(token, b"~"*len(token))
+        if out:
+            print(out.decode('utf-8'))
+        if err:
+            print(err.decode('utf-8'), file=sys.stderr)
+    return p.returncode
 
 def get_token():
     """
@@ -102,23 +127,26 @@ def get_token():
     token = token.encode('utf-8')
     return token
 
-def run(args):
+def run(args, shell=False, exit=True):
     """
     Run the command ``args``.
 
     Automatically hides the secret GitHub token from the output.
+
+    If shell=False (recommended for most commands), args should be a list of
+    strings. If shell=True, args should be a string of the command to run.
+
+    If exit=True, it exits on nonzero returncode. Otherwise it returns the
+    returncode.
     """
     if "DOCTR_DEPLOY_ENCRYPTION_KEY" in os.environ:
         token = b''
     else:
         token = get_token()
-    out, err, returncode = run_command_hiding_token(args, token)
-    if out:
-        print(out.decode('utf-8'))
-    if err:
-        print(err.decode('utf-8'), file=sys.stderr)
-    if returncode != 0:
+    returncode = run_command_hiding_token(args, token, shell=shell)
+    if exit and returncode != 0:
         sys.exit(returncode)
+    return returncode
 
 def get_current_repo():
     """
@@ -180,11 +208,7 @@ def setup_GitHub_push(deploy_repo, auth_type='deploy_key', full_key_path='github
                                     TRAVIS_PULL_REQUEST)
 
     print("Setting git attributes")
-
-    run(['git', 'config', '--global', 'user.name', "Doctr (Travis CI)"])
-    # We need a dummy email or git will fail. We use this one as per
-    # https://help.github.com/articles/keeping-your-email-address-private/.
-    run(['git', 'config', '--global', 'user.email', 'drdoctr@users.noreply.github.com'])
+    set_git_user_email()
 
     remotes = subprocess.check_output(['git', 'remote']).decode('utf-8').split('\n')
     if 'doctr_remote' in remotes:
@@ -212,18 +236,40 @@ def setup_GitHub_push(deploy_repo, auth_type='deploy_key', full_key_path='github
     print("Fetching doctr remote")
     run(['git', 'fetch', 'doctr_remote'])
 
+    return canpush
+
+def set_git_user_email():
+    """
+    Set global user and email for git user if not already present on system
+    """
+    username = subprocess.run(shlex.split('git config user.name'), stdout=subprocess.PIPE)
+    if not username.stdout:
+        run(['git', 'config', '--global', 'user.name', "Doctr (Travis CI)"])
+
+    email = subprocess.run(shlex.split('git config user.email'), stdout=subprocess.PIPE)
+    if not email.stdout:
+        # We need a dummy email or git will fail. We use this one as per
+        # https://help.github.com/articles/keeping-your-email-address-private/.
+        run(['git', 'config', '--global', 'user.email', 'drdoctr@users.noreply.github.com'])
+
+def checkout_deploy_branch(deploy_branch, canpush=True):
+    """
+    Checkout the deploy branch, creating it if it doesn't exist.
+    """
     #create empty branch with .nojekyll if it doesn't already exist
-    new_deploy_branch = create_deploy_branch(deploy_branch, push=canpush)
-    print("Checking out {}".format(deploy_branch))
-    local_deploy_branch_exists = deploy_branch in subprocess.check_output(['git', 'branch']).decode('utf-8').split()
-    if new_deploy_branch or local_deploy_branch_exists:
-        run(['git', 'checkout', deploy_branch])
-    else:
-        run(['git', 'checkout', '-b', deploy_branch, '--track', 'doctr_remote/{}'.format(deploy_branch)])
+    create_deploy_branch(deploy_branch, push=canpush)
+    print("Checking out doctr working branch tracking doctr_remote/{}".format(deploy_branch))
+    clear_working_branch()
+    run(['git', 'checkout', '-b', DOCTR_WORKING_BRANCH, '--track', 'doctr_remote/{}'.format(deploy_branch)])
     print("Done")
 
     return canpush
 
+def clear_working_branch():
+    local_branch_names = subprocess.check_output(['git', 'branch']).decode('utf-8').split()
+    if DOCTR_WORKING_BRANCH in local_branch_names:
+        run(['git', 'branch', '-D', DOCTR_WORKING_BRANCH])
+
 def deploy_branch_exists(deploy_branch):
     """
     Check if there is a remote branch with name specified in ``deploy_branch``.
@@ -250,20 +296,24 @@ def create_deploy_branch(deploy_branch, push=True):
     Return True if ``deploy_branch`` was created, False if not.
     """
     if not deploy_branch_exists(deploy_branch):
-        print("Creating {} branch".format(deploy_branch))
-        run(['git', 'checkout', '--orphan', deploy_branch])
+        print("Creating {} branch on doctr_remote".format(deploy_branch))
+        clear_working_branch()
+        run(['git', 'checkout', '--orphan', DOCTR_WORKING_BRANCH])
         # delete everything in the new ref.  this is non-destructive to existing
         # refs/branches, etc...
         run(['git', 'rm', '-rf', '.'])
-        print("Adding .nojekyll file to {} branch".format(deploy_branch))
+        print("Adding .nojekyll file to working branch")
         run(['touch', '.nojekyll'])
         run(['git', 'add', '.nojekyll'])
         run(['git', 'commit', '-m', 'Create new {} branch with .nojekyll'.format(deploy_branch)])
         if push:
-            print("Pushing {} branch to remote".format(deploy_branch))
-            run(['git', 'push', '-u', 'doctr_remote', deploy_branch])
-        # return to master branch
-        run(['git', 'checkout', '-'])
+            print("Pushing working branch to remote {} branch".format(deploy_branch))
+            run(['git', 'push', '-u', 'doctr_remote', '{}:{}'.format(DOCTR_WORKING_BRANCH, deploy_branch)])
+        # return to master branch and clear the working branch
+        run(['git', 'checkout', 'master'])
+        run(['git', 'branch', '-D', DOCTR_WORKING_BRANCH])
+        # fetch the remote so that doctr_remote/{deploy_branch} is resolved
+        run(['git', 'fetch', 'doctr_remote'])
 
         return True
     return False
@@ -288,6 +338,21 @@ def find_sphinx_build_dir():
 # TRAVIS_JOB_NUMBER = os.environ.get("TRAVIS_JOB_NUMBER", '')
 # ACTUAL_TRAVIS_JOB_NUMBER = TRAVIS_JOB_NUMBER.split('.')[1]
 
+def copy_to_tmp(source):
+    """
+    Copies ``source`` to a temporary directory, and returns the copied location.
+    """
+    tmp_dir = tempfile.mkdtemp()
+    # Use pathlib because os.path.basename is different depending on whether
+    # the path ends in a /
+    p = pathlib.Path(source)
+    new_dir = os.path.join(tmp_dir, p.name)
+    if os.path.isdir(source):
+        shutil.copytree(source, new_dir)
+    else:
+        shutil.copy2(source, new_dir)
+    return new_dir
+
 def sync_from_log(src, dst, log_file):
     """
     Sync the files in ``src`` to ``dst``.
@@ -301,8 +366,6 @@ def sync_from_log(src, dst, log_file):
     ``src``. ``added`` also includes the log file.
     """
     from os.path import join, exists, isdir
-    if not src.endswith(os.sep):
-        src += os.sep
 
     added, removed = [], []
 
@@ -321,7 +384,14 @@ def sync_from_log(src, dst, log_file):
             else:
                 print("Warning: File %s doesn't exist." % new_f, file=sys.stderr)
 
-    files = glob.iglob(join(src, '**'), recursive=True)
+    if os.path.isdir(src):
+        if not src.endswith(os.sep):
+            src += os.sep
+        files = glob.iglob(join(src, '**'), recursive=True)
+    else:
+        files = [src]
+        src = os.path.dirname(src) + os.sep
+
     # sorted makes this easier to test
     for f in sorted(files):
         new_f = join(dst, f[len(src):])
@@ -393,7 +463,7 @@ The doctr command that was run is
 
     return False
 
-def push_docs(deploy_branch='gh-pages'):
+def push_docs(deploy_branch='gh-pages', retries=3):
     """
     Push the changes to the branch named ``deploy_branch``.
 
@@ -403,10 +473,19 @@ def push_docs(deploy_branch='gh-pages'):
 
     """
 
-    print("Pulling")
-    run(['git', 'pull', 'doctr_remote', deploy_branch])
-    print("Pushing commit")
-    run(['git', 'push', '-q', 'doctr_remote', deploy_branch])
+    code = 1
+    while code and retries:
+        print("Pulling")
+        code = run(['git', 'pull', '-s', 'recursive', '-X', 'ours', 'doctr_remote', deploy_branch])
+        print("Pushing commit")
+        code = run(['git', 'push', '-q', 'doctr_remote', '{}:{}'.format(DOCTR_WORKING_BRANCH, deploy_branch)])
+        if code:
+            retries -= 1
+            print("Push failed, retrying")
+            time.sleep(1)
+        else:
+            return
+    sys.exit("Giving up...")
 
 def determine_push_rights(branch_whitelist, TRAVIS_BRANCH, TRAVIS_PULL_REQUEST):
     """Check if Travis is running on ``master`` (or a whitelisted branch) to
